define(["require", "exports", "tslib", "react", "react-dom", "@fluentui/monaco-editor", "@fluentui/react/lib/Utilities", "../transpiler/transpile", "./Editor", "../utilities/index"], function (require, exports, tslib_1, React, ReactDOM, monaco, Utilities_1, transpile_1, Editor_1, index_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.TsxEditor = void 0;
    React = tslib_1.__importStar(React);
    ReactDOM = tslib_1.__importStar(ReactDOM);
    monaco = tslib_1.__importStar(monaco);
    var typescript = monaco.languages.typescript;
    var typescriptDefaults = typescript.typescriptDefaults;
    var filePrefix = 'file:///';
    var filename = filePrefix + 'main.tsx';
    /**
     * Wrapper for rendering a Monaco instance and also transpiling/eval-ing the React example code inside.
     */
    var TsxEditor = function (props) {
        var editorProps = props.editorProps, onTransformFinished = props.onTransformFinished, compilerOptions = props.compilerOptions, _a = props.supportedPackages, supportedPackages = _a === void 0 ? index_1.SUPPORTED_PACKAGES : _a;
        // Hooks must be called unconditionally, so we have to create a backup ref here even if we
        // immediately throw it away to use the one passed in.
        var backupModelRef = React.useRef();
        var modelRef = editorProps.modelRef || backupModelRef;
        // Load the globals before loading the editor (otherwise there will be an error executing the
        // example code because the globals it depends on aren't defined)
        var hasLoadedGlobals = _useGlobals(supportedPackages);
        // Set up compiler options
        _useCompilerOptions(compilerOptions);
        // Set up type checking after globals are loaded
        var hasLoadedTypes = _useTypes(supportedPackages, hasLoadedGlobals);
        // Store the latest onChange in a ref to ensure that we get the latest values
        // without forcing re-rendering
        var onChangeRef = React.useRef();
        onChangeRef.current = function (text) {
            if (editorProps.onChange) {
                // If the consumer provided an additional onChange, call that too
                editorProps.onChange(text);
            }
            (0, transpile_1.transpileAndEval)(modelRef.current, supportedPackages).then(onTransformFinished);
        };
        // After type checking and globals are set up, call onChange to transpile
        React.useEffect(function () {
            if (hasLoadedTypes && modelRef.current) {
                onChangeRef.current(modelRef.current.getValue());
            }
        }, [onChangeRef, hasLoadedTypes, modelRef]);
        return (React.createElement(Editor_1.Editor, tslib_1.__assign({}, editorProps, { filename: filename, modelRef: modelRef, 
            // Don't track changes until types have loaded
            onChange: hasLoadedTypes ? onChangeRef.current : undefined })));
    };
    exports.TsxEditor = TsxEditor;
    function _useGlobals(supportedPackages) {
        var _a = React.useState(false), hasLoadedGlobals = _a[0], setHasLoadedGlobals = _a[1];
        React.useEffect(function () {
            setHasLoadedGlobals(false);
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            var win = (0, Utilities_1.getWindow)();
            if (!win.React) {
                win.React = React;
            }
            if (!win.ReactDOM) {
                win.ReactDOM = ReactDOM;
            }
            Promise.all(supportedPackages.map(function (group) {
                if (!win[group.globalName]) {
                    /* eslint-disable @typescript-eslint/no-explicit-any */
                    return new Promise(function (resolve) {
                        // handle either promise or callback function
                        var globalResult = group.loadGlobal(resolve);
                        if (globalResult && globalResult.then) {
                            globalResult.then(resolve);
                        }
                    }).then(function (globalModule) { return (win[group.globalName] = globalModule); });
                    /* eslint-enable @typescript-eslint/no-explicit-any */
                }
                else {
                    return undefined;
                }
            })).then(function () { return setHasLoadedGlobals(true); });
        }, [supportedPackages]);
        return hasLoadedGlobals;
    }
    function _useCompilerOptions(compilerOptions) {
        React.useEffect(function () {
            var oldCompilerOptions = typescriptDefaults.getCompilerOptions();
            typescriptDefaults.setCompilerOptions(tslib_1.__assign(tslib_1.__assign({ 
                // The compiler options used here generally should *not* be strict, to make quick edits easier
                experimentalDecorators: true, preserveConstEnums: true, 
                // implicit global `this` usage is almost always a bug
                noImplicitThis: true }, compilerOptions), { 
                // These options are essential to making the transform/eval and types code work (no overriding)
                allowNonTsExtensions: true, target: typescript.ScriptTarget.ES2015, jsx: typescript.JsxEmit.React, module: typescript.ModuleKind.ESNext, baseUrl: filePrefix, 
                // This is updated after types are loaded, so preserve the old setting
                paths: oldCompilerOptions.paths }));
        }, [compilerOptions]);
    }
    function _useTypes(supportedPackages, isReady) {
        var _a = React.useState(false), hasLoadedTypes = _a[0], setHasLoadedTypes = _a[1];
        React.useEffect(function () {
            if (!isReady) {
                return;
            }
            // Initially disable type checking
            typescriptDefaults.setDiagnosticsOptions({ noSemanticValidation: true });
            // Load types and then turn on full type checking
            _loadTypes(supportedPackages).then(function () {
                typescriptDefaults.setDiagnosticsOptions({ noSemanticValidation: false });
                setHasLoadedTypes(true);
            });
        }, [supportedPackages, isReady]);
        return hasLoadedTypes;
    }
    /**
     * Load types for React and any other packages.
     */
    function _loadTypes(supportedPackages) {
        var promises = [];
        var typesPrefix = filePrefix + 'node_modules/@types';
        // React types must be loaded first (don't use import() to avoid potential bundling issues)
        promises.push(new Promise(function (resolve) {
            return require.ensure([], function (require) {
                // raw-loader 0.x exports a single string, and later versions export a default.
                // The package.json specifies 0.x, but handle either just in case.
                var result = require('!raw-loader?esModule=false!@types/react/index.d.ts');
                typescriptDefaults.addExtraLib(typeof result === 'string' ? result : result.default, "".concat(typesPrefix, "/react/index.d.ts"));
                resolve();
            });
        }));
        // Load each package and add it to TS (and save path mappings to add to TS later)
        var pathMappings = {};
        for (var _i = 0, supportedPackages_1 = supportedPackages; _i < supportedPackages_1.length; _i++) {
            var group = supportedPackages_1[_i];
            var _loop_1 = function (pkg) {
                var packageName = pkg.packageName, loadTypes = pkg.loadTypes;
                // Get the pretend @types package name
                // (for a scoped package like @fluentui/utilities, this will be fluentui__utilities)
                var scopedMatch = packageName.match(/^@([^/]+)\/(.*)/);
                var typesPackageName = scopedMatch ? "".concat(scopedMatch[1], "__").concat(scopedMatch[2]) : packageName;
                // Call the provided loader function
                promises.push(Promise.resolve(loadTypes()).then(function (contents) {
                    var indexPath = "".concat(typesPrefix, "/").concat(typesPackageName, "/index");
                    // This makes TS automatically find typings for package-level imports
                    typescriptDefaults.addExtraLib(contents, "".concat(indexPath, ".d.ts"));
                    // But for deeper path imports, we likely need to map them back to the root index file
                    // (do still include '*' as a default in case the types include module paths--
                    // api-extractor rollups don't do this, but other packages' typings might)
                    // https://www.typescriptlang.org/docs/handbook/module-resolution.html#path-mapping
                    pathMappings[packageName + '/lib/*'] = ['*', indexPath];
                }));
            };
            for (var _a = 0, _b = group.packages; _a < _b.length; _a++) {
                var pkg = _b[_a];
                _loop_1(pkg);
            }
        }
        return Promise.all(promises).then(function () {
            // Add the path mappings
            typescriptDefaults.setCompilerOptions(tslib_1.__assign(tslib_1.__assign({}, typescriptDefaults.getCompilerOptions()), { paths: pathMappings }));
        });
    }
    exports.default = exports.TsxEditor;
});
//# sourceMappingURL=TsxEditor.js.map