define(["require", "exports", "@fluentui/react/lib/Utilities", "./exampleParser", "./transpileHelpers"], function (require, exports, Utilities_1, exampleParser_1, transpileHelpers_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.transformExample = void 0;
    var win = (0, Utilities_1.getWindow)();
    /**
     * Transform an example for rendering in a browser context (example page or codepen).
     */
    function transformExample(params) {
        var _a;
        var tsCode = params.tsCode, jsCode = params.jsCode, _b = params.id, id = _b === void 0 ? 'content' : _b, supportedPackages = params.supportedPackages, returnFunction = params.returnFunction;
        // Imports or exports will be removed since they are not supported.
        var code = (jsCode || tsCode)
            // Use .source because IE 11 doesn't support creating a regex from a regex
            .replace(new RegExp(exampleParser_1.IMPORT_REGEX.source, 'gm'), '')
            .replace(/^export /gm, '')
            .trim();
        var output = {};
        // Get info about the example's imports and exports
        var exampleInfo = (0, exampleParser_1.tryParseExample)(tsCode, supportedPackages);
        if (typeof exampleInfo === 'string') {
            // this means it's an error
            output.error = exampleInfo;
            return output;
        }
        var component = exampleInfo.component, imports = exampleInfo.imports;
        // Make a list of all the identifiers imported from each global, including converting renamed
        // identifiers ("foo as bar") to be renamed destructuring-style ("foo: bar")
        var supportedPackagesToGlobals = (0, transpileHelpers_1._supportedPackageToGlobalMap)(supportedPackages);
        var identifiersByGlobal = {};
        for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
            var imprt = imports_1[_i];
            if (imprt.packageName === 'react') {
                continue; // React is globally available and other imports from it aren't supported
            }
            var globalName = supportedPackagesToGlobals[imprt.packageName];
            identifiersByGlobal[globalName] = identifiersByGlobal[globalName] || [];
            (_a = identifiersByGlobal[globalName]).push.apply(_a, imprt.identifiers.map(function (item) { return (item.as ? "".concat(item.name, ": ").concat(item.as) : item.name); }));
        }
        var lines = [code];
        // Generate ThemeProvider wrapper stuff for the component if appropriate
        var finalComponent = component;
        if (identifiersByGlobal.FluentUIReact) {
            // If this is a Fluent UI React example, wrap in a <ThemeProvider> (adding an import for that if needed),
            // and initialize icons in case the example uses them.
            finalComponent = component + 'Wrapper';
            // If eval-ing the code, the component can't use JSX format
            var wrapperCode = returnFunction
                ? "React.createElement(ThemeProvider, null, React.createElement(".concat(component, ", null))")
                : "<ThemeProvider><".concat(component, " /></ThemeProvider>");
            lines.push('', "const ".concat(finalComponent, " = () => ").concat(wrapperCode, ";"));
            if (identifiersByGlobal.FluentUIReact.indexOf('ThemeProvider') === -1) {
                identifiersByGlobal.FluentUIReact.push('ThemeProvider');
            }
            if (identifiersByGlobal.FluentUIReact.indexOf('initializeIcons') === -1) {
                lines.unshift('// Initialize icons in case this example uses them', 'initializeIcons();', '');
                identifiersByGlobal.FluentUIReact.push('initializeIcons');
            }
        }
        // Add const destructuring for formerly-imported identifiers
        lines.unshift('');
        lines = Object.keys(identifiersByGlobal)
            .map(function (globalName) { return "const { ".concat(identifiersByGlobal[globalName].join(', '), " } = window.").concat(globalName, ";"); })
            .concat(lines);
        if (returnFunction) {
            // Wrap in function, with the right React instance as a parameter.
            // Parentheses allow the function to remain unnamed.
            lines.unshift('(function(React) {');
            lines.push("return ".concat(finalComponent, ";"));
            lines.push('})');
        }
        else {
            // Add render line
            lines.push("ReactDOM.render(<".concat(finalComponent, " />, document.getElementById('").concat(id, "'))"));
        }
        output.output = lines.join('\n');
        if (win && win.transformLogging) {
            /* eslint-disable no-console */
            console.log('TRANSFORMED:');
            console.log(output.output);
            /* eslint-enable no-console */
        }
        return output;
    }
    exports.transformExample = transformExample;
});
//# sourceMappingURL=exampleTransform.js.map