import { __assign } from "tslib";
import * as monaco from '@fluentui/monaco-editor';
import * as React from 'react';
import { CODE_FONT_FAMILY, DEFAULT_WIDTH, DEFAULT_HEIGHT } from './consts';
/**
 * Language-agnostic wrapper for a Monaco editor instance.
 */
export var Editor = function (props) {
    var _a = props.width, width = _a === void 0 ? DEFAULT_WIDTH : _a, _b = props.height, height = _b === void 0 ? DEFAULT_HEIGHT : _b, className = props.className, _c = props.code, code = _c === void 0 ? '' : _c, language = props.language, filename = props.filename, onChange = props.onChange, _d = props.debounceTime, debounceTime = _d === void 0 ? 1000 : _d, ariaLabel = props.ariaLabel, editorOptions = props.editorOptions;
    // Hooks must be called unconditionally, so we have to create a backup ref here even if we
    // immediately throw it away to use the one passed in.
    var backupModelRef = React.useRef();
    var modelRef = props.modelRef || backupModelRef;
    // Store the latest onChange and debounceTime in a ref to ensure that we get the latest values
    // (if they change at all, which they ideally shouldn't) without needing to re-create the editor
    var internalState = React.useRef();
    internalState.current = { onChange: onChange, debounceTime: debounceTime };
    var divRef = React.useRef(null);
    React.useEffect(function () {
        var model = (modelRef.current = monaco.editor.createModel(code, language, filename ? monaco.Uri.parse(filename) : undefined));
        var editor = monaco.editor.create(divRef.current, __assign(__assign({ minimap: { enabled: false }, fontFamily: CODE_FONT_FAMILY, ariaLabel: ariaLabel, accessibilityHelpUrl: 'https://github.com/Microsoft/monaco-editor/wiki/Monaco-Editor-Accessibility-Guide' }, (editorOptions || {})), { model: model }));
        // Handle changes (debounced)
        var debounceTimeout;
        editor.onDidChangeModelContent(function () {
            // Destructure these locally to get the latest values
            var _a = internalState.current, currDebounceTime = _a.debounceTime, currOnChange = _a.onChange;
            if (!currOnChange) {
                return;
            }
            if (debounceTimeout) {
                clearTimeout(debounceTimeout);
            }
            if (currDebounceTime) {
                debounceTimeout = window.setTimeout(function () { return currOnChange(model.getValue()); }, currDebounceTime);
            }
            else {
                currOnChange(model.getValue());
            }
        });
        return function () {
            clearTimeout(debounceTimeout);
            model.dispose();
            editor.dispose();
            modelRef.current = undefined;
        };
    }, [code, language, filename, modelRef, internalState, ariaLabel, editorOptions]);
    return React.createElement("div", { ref: divRef, style: { width: width, height: height }, className: className });
};
//# sourceMappingURL=Editor.js.map