import { __assign } from "tslib";
import * as React from 'react';
import { EditorErrorBoundary } from './EditorErrorHandler';
import { TypeScriptSnippet } from './TypeScriptSnippet';
import { EditorLoading } from './EditorLoading';
import { isEditorSupported } from '../utilities/index';
import { DEFAULT_HEIGHT } from './consts';
var TsxEditorLazy = React.lazy(function () {
    // Theoretically we could use import() here, but that pulls things into bundles when using
    // commonjs modules due to the way import is transpiled for commonjs
    // https://github.com/webpack/webpack/issues/5703#issuecomment-357512412
    return new Promise(function (resolve) { return require.ensure([], function (require) { return resolve(require('./TsxEditor')); }); });
});
export var EditorWrapper = function (props) {
    var code = props.code, _a = props.previewAs, Preview = _a === void 0 ? EditorPreview : _a, editorClassName = props.editorClassName, _b = props.previewProps, previewProps = _b === void 0 ? {} : _b, _c = props.height, height = _c === void 0 ? DEFAULT_HEIGHT : _c, width = props.width, editorAriaLabel = props.editorAriaLabel, modelRef = props.modelRef, useEditor = props.useEditor, supportedPackages = props.supportedPackages, onTransformFinishedFromProps = props.onTransformFinished, children = props.children;
    var _d = React.useState({}), transformResult = _d[0], setTransformResult = _d[1];
    var ExampleComponent = transformResult.component;
    // Check if editing should be enabled
    var canEdit = React.useMemo(function () { return isEditorSupported(code, supportedPackages, useEditor); }, [useEditor, code, supportedPackages]);
    var onTransformFinished = React.useCallback(function (result) {
        setTransformResult(result);
        if (onTransformFinishedFromProps) {
            onTransformFinishedFromProps(result);
        }
    }, [onTransformFinishedFromProps]);
    return (React.createElement("div", null,
        React.createElement("div", { className: editorClassName }, canEdit ? (
        // Editing supported -- render editor module (or loading spinner)
        React.createElement(React.Suspense, { fallback: React.createElement(EditorLoading, { height: height }) },
            React.createElement(TsxEditorLazy, { editorProps: { code: code, width: width, height: height, modelRef: modelRef, ariaLabel: editorAriaLabel }, onTransformFinished: onTransformFinished, supportedPackages: supportedPackages }))) : (
        // Editing not supported
        React.createElement(TypeScriptSnippet, null, code))),
        React.createElement(EditorErrorBoundary, { transformResult: transformResult },
            React.createElement(Preview, __assign({}, previewProps), ExampleComponent ? React.createElement(ExampleComponent, null) : children))));
};
var EditorPreview = function (props) {
    return React.createElement("div", __assign({}, props));
};
//# sourceMappingURL=EditorWrapper.js.map