import { memoizeFunction } from '@fluentui/react/lib/Utilities';
export function _getErrorMessages(errors, text) {
    var lineStarts = _getLineStarts(text);
    return errors.map(function (error) {
        if (error.messageText && typeof error.messageText === 'object') {
            // This is a multi-line ts.DiagnosticMessageChain (not sure if this happens, but handling per typings)
            error.code = error.messageText.code;
            error.messageText = error.messageText.messageText;
        }
        if (typeof error.start === 'number') {
            var lineInfo = _getErrorLineInfo(error, lineStarts);
            return "Line ".concat(lineInfo.line, " - ").concat(error.messageText, " (TS").concat(error.code, ")");
        }
        else {
            return error.messageText;
        }
    });
}
export function _getLineStarts(text) {
    var lineStarts = [0];
    var eol = /\r?\n/g;
    var match;
    while ((match = eol.exec(text))) {
        lineStarts.push(match.index + match[0].length);
    }
    return lineStarts;
}
export function _getErrorLineInfo(error, lineStarts) {
    var line = 1;
    for (; line < lineStarts.length; line++) {
        if (lineStarts[line] > error.start) {
            break;
        }
    }
    return { line: line, col: error.start - lineStarts[line - 1] + 1 };
}
/** Convert from IPackageGroup[] to a map from package name to global name. @internal */
export var _supportedPackageToGlobalMap = memoizeFunction(function (supportedPackages) {
    var packagesToGlobals = {};
    for (var _i = 0, supportedPackages_1 = supportedPackages; _i < supportedPackages_1.length; _i++) {
        var group = supportedPackages_1[_i];
        for (var _a = 0, _b = group.packages; _a < _b.length; _a++) {
            var pkg = _b[_a];
            packagesToGlobals[pkg.packageName] = group.globalName;
        }
    }
    return packagesToGlobals;
});
//# sourceMappingURL=transpileHelpers.js.map