var fabricGroup = {
    globalName: 'FluentUIReact',
    // Theoretically we could use import() here, but that pulls things into bundles when using
    // commonjs modules due to the way import is transpiled for commonjs
    // https://github.com/webpack/webpack/issues/5703#issuecomment-357512412
    loadGlobal: function (cb) { return require.ensure([], function (require) { return cb(require('@fluentui/react')); }); },
    packages: [],
};
var hooksGroup = {
    globalName: 'FluentUIReactHooks',
    loadGlobal: function (cb) { return require.ensure([], function (require) { return cb(require('@fluentui/react-hooks')); }); },
    packages: [],
};
var exampleDataGroup = {
    globalName: 'FluentUIExampleData',
    loadGlobal: function (cb) { return require.ensure([], function (require) { return cb(require('@fluentui/example-data')); }); },
    packages: [],
};
var chartingGroup = {
    globalName: 'FluentUIReactCharting',
    loadGlobal: function (cb) { return require.ensure([], function (require) { return cb(require('@fluentui/react-charting')); }); },
    packages: [],
};
var typesContext;
try {
    // Other packages' typings are copied into dist/types by a build step.
    // Load all of those typings dynamically (lazy-once puts them in a chunk together).
    typesContext = require.context('!raw-loader?esModule=false!@fluentui/react-monaco-editor/dist/types', false, /.*\.d\.ts$/, 'lazy-once');
}
catch (ex) {
    // We're probably running in jest, which doesn't have webpack's require.context
}
var loadTypes = function () { return ''; };
if (typesContext) {
    typesContext.keys().forEach(function (dtsPath) {
        // The api-extractor .d.ts rollups use the package's unscoped name (such as "utilities")
        // as the filename.
        // (example path: '!raw-loader?esModule=false!@fluentui/react-monaco-editor/dist/types/utilities.d.ts')
        var unscopedName = dtsPath.match(/\/(.*?)\.d\.ts$/)[1];
        var packageName = "@fluentui/".concat(unscopedName);
        var packageGroup = packageName === '@fluentui/example-data'
            ? exampleDataGroup
            : packageName === '@fluentui/react-hooks'
                ? hooksGroup
                : fabricGroup;
        packageGroup.packages.push({
            packageName: packageName,
            loadTypes: function () {
                // raw-loader 0.x exports a single string, and later versions export a default.
                // The package.json specifies 0.x, but handle either just in case.
                return typesContext(dtsPath).then(function (result) {
                    return typeof result === 'string' ? result : result.default;
                });
            },
        });
    });
}
else {
    // Use some defaults for jest tests (real types won't be loaded)
    fabricGroup.packages.push({ packageName: '@fluentui/date-time-utilities', loadTypes: loadTypes }, { packageName: '@fluentui/dom-utilities', loadTypes: loadTypes }, { packageName: '@fluentui/font-icons-mdl2', loadTypes: loadTypes }, { packageName: '@fluentui/foundation-legacy', loadTypes: loadTypes }, { packageName: '@fluentui/merge-styles', loadTypes: loadTypes }, { packageName: '@fluentui/react', loadTypes: loadTypes }, { packageName: '@fluentui/react-focus', loadTypes: loadTypes }, { packageName: '@fluentui/react-window-provider', loadTypes: loadTypes }, { packageName: '@fluentui/style-utilities', loadTypes: loadTypes }, { packageName: '@fluentui/theme', loadTypes: loadTypes }, { packageName: '@fluentui/utilities', loadTypes: loadTypes });
    hooksGroup.packages.push({ packageName: '@fluentui/react-hooks', loadTypes: loadTypes });
    exampleDataGroup.packages.push({ packageName: '@fluentui/example-data', loadTypes: loadTypes });
}
// Add charting package only if we're on the charting page on the demo site.
// There is an issue where the export to codepen button is not showing intermittently for charting examples.
// Adding this check while we investigate the issue. https://github.com/microsoft/fluentui/issues/29761
if (typeof window === 'object' && window.location.href.indexOf('chart') !== -1) {
    chartingGroup.packages.push({ packageName: '@fluentui/react-charting', loadTypes: loadTypes });
}
/**
 * Default supported packages for imports: `@fluentui/react` and everything it exports,
 * plus `@fluentui/example-data`. (React is implicitly supported.)
 */
export var SUPPORTED_PACKAGES = [fabricGroup, hooksGroup, exampleDataGroup, chartingGroup];
//# sourceMappingURL=defaultSupportedPackages.js.map